local M = {}

local helper = require('scenario/scenariohelper')
local logTag = 'hirochiF8races94'

local finalWaypointName = 'derby_wp12'
local finalTriggerName = 'trigger_finish1'
local playerInstance = 'scenario_player0'
local aiInstance1 = 'SunburstSammy'
local aiInstance2 = 'PickupPete'
local aiInstance3 = 'ETKiEddy'
local aiInstance4 = 'LisaLeGran'
local aiInstance5 = 'DriftMarshal'
local aiInstance6 = 'PessimaPayne'
local aiInstance7 = 'StepDemon'
local aiInstance8 = 'CovetConnie'
local aiInstance9 = 'ThunderBuck'

local aiFinishCount = 0
local finishRanks = {'1st', '2nd', '3rd', '4th', '5th', '6th', '7th', '8th', '9th', '10th'}
local running = false
local playerWon = false
local playerFinished = false
local aiFinished = false
local noOfLaps = 4
local noOfAI = 9
local lapsCompleted = {}

local extraText = { -- lines of text to use for scenario fail or win screens
	second = "Almost! Very good driving skills. You",
	third = "Not bad, a podium shower is secure. You",
	noPodium = "Ah well, no podium but at least in one piece. You",
	last = "Last, but not least... You"
}

local function reset()
  aiFinishCount = 0
  lapsCompleted[playerInstance] = 0
  lapsCompleted[aiInstance1] = 0
  lapsCompleted[aiInstance2] = 0
  lapsCompleted[aiInstance3] = 0
  lapsCompleted[aiInstance4] = 0
  lapsCompleted[aiInstance5] = 0
  lapsCompleted[aiInstance6] = 0
  lapsCompleted[aiInstance7] = 0
  lapsCompleted[aiInstance8] = 0
  lapsCompleted[aiInstance9] = 0
  
  running = false
  playerWon = false
  playerFinished = false
  aiWon = false
end

local racers = {aiInstance1,aiInstance2,aiInstance3,aiInstance4,aiInstance5,aiInstance6,aiInstance7,aiInstance8,aiInstance9}
local filenames = {"f8mc-1sunburstofforad", "f8mc-2pickupmarauder", "f8mc-3etki300m", "f8mc-4legranluxev6", "f8mc-5fullsizedriftmissile", "f8mc-6pessimazxblue", "f8mc-7coupe", "f8mc-8covetgreenrally", "f8mc-9bluebucklila"}

local function onRaceStart()

  scenario_scenarios.trackVehicleMovementAfterDamage(playerInstance, {waitTimerLimit=10})

  running = true
end

local function onCountdownStarted()
  reset()
  
  local recordings = {}
    for i, v in ipairs(racers) do
        local id = scenetree.findObject(racers[i]):getId()
        local filename = "/replays/scriptai/tracks/"..filenames[i]..".track.json"
        local scriptData = jsonReadFile(filename)
        recordings[id] = scriptData.recording
        scenetree.findObject(racers[i]):queueLuaCommand("ai.startFollowing("..serialize(recordings[id])..")")
    end
end

local function onRaceWaypointReached(data)
  if data.waypointName == finalWaypointName then
    lapsCompleted[playerInstance] = lapsCompleted[playerInstance] + 1
	if lapsCompleted[playerInstance] == (noOfLaps - 1) then
	  helper.flashUiMessage('Last Lap!', 5)
	end
    if lapsCompleted[playerInstance] == noOfLaps then
	  playerFinished = true
      if not aiFinished then
	    helper.flashUiMessage('Finish! Wow 1st!', 2)
        playerWon = true
	  elseif playerFinished then
	    helper.flashUiMessage('Finish!', 2)
      end
    end
  end
end

local function onRaceResult()
  local text = ' finished the race in '..finishRanks[aiFinishCount + 1]..' place. '
  if playerWon then
    text = 'Checkered Flag! Wow you '..text..'Congratulations!'
    scenario_scenarios.finish({msg = text})
  elseif aiFinishCount == 1 then
    scenario_scenarios.finish({msg = extraText.second..' '..text})
  elseif aiFinishCount == 2 then
    scenario_scenarios.finish({msg = extraText.third..' '..text})
  elseif aiFinishCount == noOfAI then
    scenario_scenarios.finish({msg = extraText.last..' '..text})
  elseif aiFinishCount ~= 1 and aiFinishCount ~= 2 and aiFinishCount ~= noOfAI then
    scenario_scenarios.finish({msg = extraText.noPodium..' '..text})
  end
end

local function onBeamNGTrigger(data)
  if data.triggerName == finalTriggerName and data.event == 'enter' then
    if data.subjectName ~= playerInstance then
      lapsCompleted[data.subjectName] = lapsCompleted[data.subjectName] + 1
      if lapsCompleted[data.subjectName] == noOfLaps then
	    scenetree.findObject(data.subjectName):queueLuaCommand('ai.stopFollowing()')
	    if not playerFinished then
		  helper.flashUiMessage(tostring(data.subjectName)..' finished the race in '..finishRanks[aiFinishCount + 1]..' place. ', 3)
          aiFinished = true
          aiFinishCount = aiFinishCount + 1
		end
      end
    end
  end
end

M.onRaceStart = onRaceStart
M.onCountdownStarted = onCountdownStarted
M.onRaceWaypointReached = onRaceWaypointReached
M.onRaceResult = onRaceResult
M.onBeamNGTrigger = onBeamNGTrigger
return M